body{
    color:white;
    font-weight:normal;
}

.action{
    color:magenta;
    font-weight:bold;
}

.input{
    color:white;
    font-weight:bold;
}

.prompt{
    color:red;
}

.prefix{
    color:blue;
    font-weight:bold;
}

.notification .label{
    color:white;
    font-weight:bold;
}


.notification .footer{
    color:white;
    font-weight:normal;
}


.string .label{
    color:white;
}

.string .default-wrapper{
    color:white;
    font-weight:bold;
}

.string .default-value{
    color:yellow;
}

.integer .label{
    color:white;
}

.integer .default-wrapper{
    color:white;
    font-weight:bold;
}

.integer .default-value{
    color:yellow;
}

.boolean .label{
    color:white;
}

.boolean .default-wrapper{
    color:white;
    font-weight:bold;
}

.boolean .default-value{
    color:yellow;
}

.boolean .other-value{
    color:yellow;
}

.boolean .seperator{
    color:white;
    font-weight:bold;
}

.choices .label{
    color:white;
}

.choices .default-wrapper{
    color:white;
    font-weight:bold;
}

.choices .default-value{
    color:yellow;
}

.choices .option-key{
    color:yellow;
}

.choices .seperator{
    color:yellow;
    font-weight:lighter;
}

.choices .option-value{
    color:white;
    font-weight:bold;
}

.choices .action{
    color:magenta;
    font-weight:bold;
}

.choices .selection{
    color:white;
}
